@@ -31,6 +31,6 @@ TimeOrderedRecords <- function(x) {
 
 getTORnames <- function(obj) obj@TOR.columns
 
-getTimeID <- function(obj) as.data.frame(obj)[, obj@TOR.columns]
+getTimeID <- function(obj) as.data.frame(obj)[, getTORnames(obj)]
 #####################################################################################
 #####################################################################################
